import { AndroidResolvedVersions, BundlingResult, NativePlatform, PackagePath } from 'ern-core';
import { ContainerGenerator, ContainerGeneratorConfig, ContainerGenResult } from 'ern-container-gen';
export interface AndroidDependencies {
    files: string[];
    transitive: string[];
    raw: string[];
    regular: string[];
    annotationProcessor: string[];
}
export declare enum JavaScriptEngine {
    HERMES = 0,
    JSC = 1
}
export default class AndroidGenerator implements ContainerGenerator {
    get name(): string;
    get platform(): NativePlatform;
    generate(config: ContainerGeneratorConfig): Promise<ContainerGenResult>;
    doesDirectoryContainsKotlinSourceFiles(dir: string): Promise<boolean>;
    fillContainerHull(config: ContainerGeneratorConfig): Promise<void>;
    postBundle(config: ContainerGeneratorConfig, bundle: BundlingResult, reactNativeVersion: string): Promise<void>;
    getJavaScriptEngine(config: ContainerGeneratorConfig): JavaScriptEngine;
    /**
     * Starting with React Native 0.60.0, JavaScriptCore engine is distributed
     * separately from React Native and comes in two variants :
     * 'android-jsc' and 'android-jsc-intl'.
     *
     * More details@ https://github.com/react-native-community/jsc-android-buildscript
     *
     * Prior to React Native 0.60.0, the 'libjsc.so' native library files were
     * shipped inside React Native AAR itself. This is not the case anymore.
     * The 'libjsc.so' files are now distributed in the 'jsc-android' npm package,
     * inside an aar library.
     *
     * This function retrieve the 'jsc-android' package and unzip the AAR
     * matching the desired JSC variant ('android-jsc' or 'android-jsc-intl').
     * It then copy the 'libjsc.so' files to the 'jniLibs' directory of the
     * Container. This way, the JSC engine is shipped within the Container and
     * applications won't crash at runtime when trying to load this library.
     */
    injectJavaScriptCoreEngine(config: ContainerGeneratorConfig, reactNativeVersion: string): Promise<void>;
    /**
     * Inject hermes engine into the Container
     * Done in a similar way as injectJavaScriptCoreEngine method
     */
    injectHermesEngine(config: ContainerGeneratorConfig, reactNativeVersion: string): Promise<void>;
    buildImplementationStatements(dependencies: AndroidDependencies, androidVersions: AndroidResolvedVersions): any[];
    highestVersion(d: string[]): string;
    addAndroidPluginHookClasses(plugins: PackagePath[], outDir: string): Promise<any>;
    /**
     * Convert files in a directory from support library to AndroidX
     * eg: import android.support.annotation.NonNull => import androidx.annotation.NonNull
     */
    convertToAndroidX(dir: string): number;
    buildAndroidPluginsViews(plugins: PackagePath[], mustacheView: any): Promise<any>;
}
